from pyramid.exceptions import ConfigurationError
from pyramid.settings import asbool
from pymongo import Connection

URI = 'mongo.uri'
MONGOENGINE = 'mongo.mongoengine'
USERNAME = 'mongo.username'
PASSWORD = 'mongo.password'
DBNAME = 'mongo.db'
GREENLETS = 'mongo.use_greenlets'

def get_connection(config, conn_cls=None):
    """get_connection creates a connection to one or more mongodb server. 
       It take as argument a config that should have the "mongo.uri" set 
       to a string separated by new lines for each server. 

       The uri must be of a form acceptable by mongodb. 
       http://www.mongodb.org/display/DOCS/Connections
    """

    if conn_cls is None:
        conn_cls = Connection
        
    registry = config.registry

    uri = registry.settings.get(URI)
    greenlets = registry.settings.get(GREENLETS)

    if uri is None:
        raise ConfigurationError('There is no configured "mongo.uri"')

    # Spliting configs to get more than one uri
    if not isinstance(uri, list):
        uri = uri.splitlines()

    kargs = {
        'use_greenlets':  asbool(greenlets)
    }

    return conn_cls(uri, **kargs)

def get_db(request, name=None):
    """get_db opens a handle for a database using a connection.
       the primary database is defined by the setting "mongo.db".

       If passed "name" as argument, get_db will return a different 
       database than the one set in the settings. 

       If you have mongo.username and mongo.password set, it will try
       to connecto to the database.

       Here is an example
       
       mongo.uri = 127.0.0.1
                   localhost
       mongo.db = blog

       mongo.username.blog = theuser
       mongo.password.blog = thepassword

       mongo.username.blog2 = theuser2
       mongo.password.blog2 = thepassword2
    """

    dbname = name
    registry = request.registry

    if name is None:
        dbname = registry.settings.get(DBNAME)

    if dbname is None:
        raise ConfigurationError('There is no defined database name')

    mongodbs = getattr(request, '_mongo_dbs', dict())

    db = mongodbs.get(dbname)

    if db is None:
        conn = getattr(registry, '_mongo_conn', None)

        if conn is None:
            raise ConfigurationError(
                'There is no database connection available')

        db = conn[dbname]

        mongodbs[dbname] = db
        request._mongo_dbs = mongodbs

    username = registry.settings.get(USERNAME + '.' + dbname)
    password = registry.settings.get(PASSWORD + '.' + dbname)

    if not username is None and not password is None:
        db.authenticate(username, password)

    def end_request(request):
        db.logout()
        db.connection.end_request() 

    request.add_finished_callback(end_request)

    return db

def setup_mongoengine(config):
    # Simple setup mongoengine 
    from mongoengine import connection
    print "Loading mongoengine"
    registry = config.registry

    connection._connections['default'] = config.registry._mongo_conn
    connection._connection_settings['default'] = {
        'name': registry.settings.get(DBNAME),
        'username': registry.settings.get(USERNAME),
        'password': registry.settings.get(PASSWORD),
    }

def includeme(config, get_connection=get_connection):
    # get_connection passed for testing
    """
        Get a mongodb instance from the URI in the config file
        mongodb.uri
    """
    config.registry._mongo_conn = get_connection(config)
    
    mongoengine = asbool(config.registry.settings.get(MONGOENGINE))
    if mongoengine and config.registry._mongo_conn:
        setup_mongoengine(config)



from setuptools import setup, find_packages

version = '0.1.1'

testing_extras = ['nose', 'coverage']
docs_extras = ['Sphinx']

setup(name='pyramid_mongo',
      version=version,
      description="Mongodb support for pyramid",
      long_description="""\
        This plugin lets you create a connection to a mongodb server and query its database.
        It currently only support one database, one host and authentication
""",
      classifiers=[],
      keywords='pymongo mongodb pyramid',
      author='Lo\xc3\xafc Faure-Lacroix',
      author_email='lamerstar@gmail.com',
      url='delicieuxgateau.ca',
      license='',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      test_suite="pyramid_mongo",
      install_requires=[
          # -*- Extra requirements: -*-
          'pyramid',
          'pymongo',
      ],
      extras_require = {
          'dev':testing_extras,
          'docs':docs_extras,
          },
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



from pyramid import testing
from unittest import TestCase
from pyramid.exceptions import ConfigurationError
from pymongo.errors import AutoReconnect

class Test_get_connection(TestCase):

    def setUp(self):
        self.config = testing.setUp()

    def tearDown(self):
        testing.tearDown()

    def _callFUT(self, config, conn_cls=None):
        from pyramid_mongo import get_connection
        return get_connection(config, conn_cls)

    def test_without_include(self):
         self.assertRaises(ConfigurationError, self._callFUT, self.config)

    def test_without_uri(self):
        self.assertRaises(ConfigurationError, self._callFUT, self.config)
        
    def test_with_invalid_uri(self):
        from pymongo.errors import ConfigurationError
        self.config.registry.settings['mongo.uri'] = "1"
        self.assertRaises(AutoReconnect, self._callFUT, self.config)
        self.config.registry.settings['mongo.uri'] = ""
        self.assertRaises(ConfigurationError, self._callFUT, self.config)

    def test_with_valid_uri(self):
        self.config.registry.settings['mongo.uri'] = "mongodb://localhost/"
        conn = self._callFUT(self.config, DummyConnection)
        self.assertEqual(conn.uri[0], 'mongodb://localhost/')

class Test_get_db(TestCase):

    def _callFUT(self, request, dbname=None):
        from pyramid_mongo import get_db
        return get_db(request, name=dbname)

    def _makeRequest(self):
        request = testing.DummyRequest()
        # other things
        request._mongo_dbs = dict()
        request.registry.settings = dict()
        return request

    def test_without_name(self):
        request = self._makeRequest()
        #del request.registry._mongo_conn
        self.assertRaises(ConfigurationError, self._callFUT, request)
    
    def test_without_conn_with_name(self):
        request = self._makeRequest()

        if hasattr(request.registry, '_mongo_conn'):
            delattr(request.registry, '_mongo_conn')

        self.assertRaises(ConfigurationError, self._callFUT, request, 'fun')

    def test_with_existing_db(self):
        request = self._makeRequest()
        request._mongo_dbs = dict(test=DummyDB())
        db = self._callFUT(request, 'test')
        self.assertEqual(db, request._mongo_dbs['test'])

    def test_with_new_db(self):
        request = self._makeRequest()
        request.registry._mongo_conn = DummyConnection()
        db = self._callFUT(request, 'test')
        db2 = request._mongo_dbs['test']
        self.assertEqual(db, db2)

    def test_auth(self):
        request = self._makeRequest()
        request.registry._mongo_conn = DummyConnection()
        
        request.registry.settings['mongo.username.test'] = 'fun'
        request.registry.settings['mongo.password.test'] = 'fusdn'
        db = self._callFUT(request, 'test')
        self.assertEqual(db.authenticated, False)

        request.registry.settings['mongo.username.test'] = 'admin'
        request.registry.settings['mongo.password.test'] = 'fun'
        db = self._callFUT(request, 'test')
        self.assertEqual(db.authenticated, True)

    def test_endrequest_callback(self):
        request = self._makeRequest()
        conn = DummyConnection()
        request.registry._mongo_conn = conn
        db = self._callFUT(request, 'test')
        db.connection = conn
        request.finished_callbacks[0](request)
        self.assertTrue(db.logged_out)
        self.assertTrue(conn.ended)

class Test_includeme(TestCase):
    def _callFUT(self, config, get_connection):
        from pyramid_mongo import includeme
        return includeme(config, get_connection)

    def setUp(self):
        self.config = testing.setUp()

    def tearDown(self):
        testing.tearDown()

    def test_with_normal_config(self):
        def get_connection(request):
            return 'conn'
        self.config.registry.settings['mongo.uri'] = 'localhost'
        self.config.registry.settings['mongo.db'] = 'blog'
        self._callFUT(self.config, get_connection)
        self.assertEqual(self.config.registry._mongo_conn, 'conn')

class DummyDB(object):

    def authenticate(self, username, password):
        self.authenticated = username == 'admin' and password == 'fun'
        return self.authenticated

    def logout(self):
        self.logged_out = True

        

class DummyConnection(object):
    def __init__(self, uri=None):
        self.uri = uri

    def __getitem__(self, name):
        return DummyDB()

    def end_request(self):
        self.ended = True
        
